import processing.serial.*;    // Arduinoと通信するためのライブラリを読み込む

Serial port;                   // シリアル通信を行うための変数の定義
float ax, ay;                  // ボールの加速度
float vx, vy;                  // 速度
float px, py;                  // 位置

void setup()
{
    size(255, 255);                         // 255×255ドットの画面を作成
    port = new Serial(this, "COM3", 9600);  // 通信ポートと速度の設定
    textSize(24);                           // 文字のサイズを24pt
    textAlign(CENTER);                      // 文字を中央に配置
    port.clear();                           // 受信データをクリア
    px = width/2;                           // ボールの初期位置
    py = height/2;
}

void draw()
{
    background(0);                 // 画面を黒で塗りつぶして更新
    int cx, cy, c;
    if (port.available() > 2) {    // データが3以上送られて来たら
        c = port.read();
        if (c == 'a') {            // 読み込んだデータが開始合図か？
            cx = port.read();      // x方向のデータの読み込み
            cy = port.read();      // y方向
            ax = (cx-127)-vx*0.1;  // 加速度に変換
            ay = (127-cy)-vy*0.1;
        }
    }
    vx += ax*0.1;                  // 速度の計算
    vy += ay*0.1;
    px += vx*0.1;                  // 位置の計算
    py += vy*0.1;
    fill(255);                     // 土俵の表示
    ellipse(width/2, height/2, 200, 200);
    fill(127);                     // ボールの表示
    ellipse(px, py, 10, 10);

    if ((px-width/2)*(px-width/2)+(py-height/2)*(py-height/2) > 100*100) {    // 土俵から出たら
        text("Game Over", width/2, height/2);
        text("Press [s]", width/2, height/2+30);
    }
}

void keyPressed()
{
    if (key == 's') {              // 「s」キーが押されたら
        px = width/2;              // 初期位置に戻す
        py = height/2;
        vx = 0;
        vy = 0;
    }
}

