import processing.serial.*;    // Arduinoと通信するためのライブラリを読み込む
import processing.video.*;     // カメラを使うためのライブラリを読み込む

Serial port;                   // シリアル通信を行うための変数の定義
Capture video;                 // カメラを使うための変数の定義
float angle;                   // サーボモーターの角度を保存する変数の定義

void setup() {
    size(640, 480);                         // 640×480ドットの画面を作成
    video = new Capture(this, 640, 480);    // カメラの設定
    video.start();                          // カメラからの情報取得をスタート
    port = new Serial(this, "COM3", 9600);  // 通信ポートと速度の設定
    angle = 60;                             // 初期角度を60度
}

void draw() {
// カメラ画像を読み込み，赤い部分を検出
if (video.available() == true) {  // カメラの使う準備ができていれば画像を読み込む
    video.read();
    }
    image(video, 0, 0);           // カメラ画像を画面に表示
    video.loadPixels();           // カメラ画像を解析するためにピクセル単位で読み込む
    int index = 0;
    int rx = 0, ry = 0, rn = 0;
    for (int y=0; y<video.height; y++) {    // 全ピクセルを確認して赤と認識するピクセルの位置と数を調べる
        for (int x=0; x<video.width; x++) {
            float r = red(video.pixels[index]);
            float g = green(video.pixels[index]);
            float b = blue(video.pixels[index]);
            if (r > 150 && g < 120 && b < 130) {  // RGB値の範囲を区切ることで赤を認識
                stroke(255, 0, 0);         // 赤と認識した位置に赤い点を描く
                point(x, y);
                rn++;
                rx += x;
                ry += y;
            }
            index++;
        }
    }
    // 赤い部分の平均値を求めサーボモーターの角度を決める部分
    if (rn > 0) {                       // 赤と認識した位置の平均を計算
        rx /= rn;
        ry /= rn;
        fill(255, 204, 0, 128);         // 平均位置に半透明の円を描く
        ellipse(rx, ry, 100, 100);
        if (rx > width/2+5) {           // 画面の中心より5ピクセル以上右にあれば
            angle -= 0.5;               // サーボモーターの角度を小さく
            if (angle < 0)angle = 0;
        } else if (rx < width/2-5) {    // 画面の中心より5ピクセル以上左にあれば
            angle += 0.5;               // サーボモーターの角度を大きく
            if (angle > 120)angle = 120;
        }
        port.write(byte(angle));        // サーボモーターの角度を送る
    }
    // クリックしたピクセルのRGB値をコンソールに表示する部分（おまけの機能）
    if (mousePressed == true) {
        index = mouseX+video.width*mouseY;
        float r = red(video.pixels[index]);
        float g = green(video.pixels[index]);
        float b = blue(video.pixels[index]);
        println(r + "," + g + "," + b);
    }
}

