import processing.serial.*;    // Arduinoと通信するためのライブラリを読み込む

Serial port;                      // シリアル通信を行うための変数の定義
PrintWriter writer;               // ファイルを書き出すための変数を定義
float [] temp = new float [255];  // 温度を保存するための変数を定義

void setup()
{
    size(255, 150);                         // 255x150ドットの画面を作成
    port = new Serial(this, "COM3", 9600);  // 通信ポートと速度の設定
    background(255);                        // 背景を白
    fill(0);                                // 文字の色を黒
    stroke(0);                              // 線の色を黒
    port.clear();                           // 受信データをクリア
    writer = createWriter("data.txt");      // データを保存するためのファイル設定
    for(int i=0; i<255; i++)
        temp[i] = 0;
}

void draw()
{
    int ns = second();                      // 現在の時刻（秒）
    int nm = minute();                      // 分
    int nh = hour();                        // 時

    if(port.available() > 2){               // データが3つ以上送られて来たか？
        int c = port.read();                // データを読み込む
        if(c == 'a'){                       // 「a」ならば
            int c1 = port.read();           // 1000の位と100の位のデータを読み込む
            int c2 = port.read();           // 10の位と1の位のデータを読み込む
            int c3 = c1*100+c2;             // 元のデータを復元
            float t = (float)(c3) / 1023.0 * 5 * 100;
                // 温度データに変換
            writer.println(nh + "\t" + nm + "\t" + ns + "\t" + nf(t, 1, 1));  // ファイルに保存
            background(255);                // 背景を白で塗りつぶして更新
            text("時刻：" + nh + "時" + nm + "分" + ns + "秒", 10, 20);    // 画面に時刻を表示
            text("温度：" + nf(t, 1, 1) + "度", 150, 20);      // 画面に温度を表示

    for(int i=0; i<254; i++)
        temp[i] = temp[i+1];
        temp[254] = t;
        for(int i=0; i<254; i++)
            line(i, height-10-temp[i], i+1, height-10-temp[i+1]);
        }
    }
}

void exit() {
    writer.flush();        // バッファに残っているデータをファイルに書き込む
    writer.close();        // ファイルを閉じる

    super.exit();          // Processingの本来の終了処理（必ず必要）
}

