import processing.serial.*;    // Arduinoと通信するためのライブラリを読み込む

Serial port;                   // シリアル通信を行うための変数の定義
byte [] FontData;              // 美咲フォントの解析用
int [] BlockStart;
int [] BlockEnd;
int Tnum;
int [] WordData;               // 文字データの表示用
int WordDataPos;
int WordDataLen;
int WordSpeed;

void setup() {
    size(255, 255);                         // 255×255ドットの画面を作成
    port = new Serial(this, "COM3", 9600);  // 通信ポートと速度の設定
    background(255);                        // 背景を白
    fill(0);                                // 文字の色を黒
    frameRate(10);                          // 画面の更新頻度を10fps（0.1秒ごと）

    ReadFont();

    WordData = new int [8*100];             // 最大100文字
    WordDataPos = 0;                        // 表示しているデータ
    WordDataLen = 0;                        // 最大のデータ
    WordSpeed = 0;                          // 文字の切り替わりタイミング
}

void draw()
{
    int i;
    // 文字の送信
    if (WordDataLen > 0) {                   // 文字が読み込まれているか？
        WordSpeed++;
        if (WordSpeed == 10) {               // 1秒経過したか？
            WordSpeed = 0;
            for (i=0; i<8; i++) {            // 8個の文字データを送る
                port.write(WordData[WordDataPos+i]);
            }
            WordDataPos += 8;                // 現在送った文字データの位置を更新
            if (WordDataLen <= WordDataPos)  // 最大文字データ数より大きい場合
                WordDataPos = 0;             // 初めから表示させるために初期化
        }
    }
    // 文字データの読み込み
    if (mousePressed == true) {              // マウスが押されたか？
        background(255);                     // 画面を更新
        byte b[] = loadBytes("word.txt");    // 文字データをファイルから読み込む
        String str = new String(b);

        PFont myFont;                        // 画面に表示するフォントを設定
        textAlign(CENTER);
        myFont = createFont("MSゴシック", 24);
        textFont(myFont);
        text(str, width/2, height/2);
        WordDataLen = 0;
        for (i=0; i<b.length; i+=2) {        // 読み込んだ文字データから美咲フォントの中で該当する番号をすべての文字について探す。
            int code = SearchFont((b[i] & 0xFF)*0x100+(b[i+1] & 0xFF));
            DrawFont(code);
        }
    }
}

// 美咲フォントを読み込む
void ReadFont()
{
    FontData = loadBytes("MISAKI.FNT");

    Tnum = FontData[17] & 0xff;
    BlockStart = new int [Tnum];
    BlockEnd = new int [Tnum];
    int i;
    for (i=0; i<Tnum; i++) {
        BlockStart[i] = ((FontData[19+i*4]&0xff)<<8) + (FontData[18+i*4]&0xff);
        BlockEnd[i] = ((FontData[21+i*4]&0xff)<<8) + (FontData[20+i*4]&0xff);
    }
}

// 文字データを美咲フォント上のデータに変換する
int SearchFont(int code)
{
    println("code = " + hex(code, 4));
    int i;
    int n = 0;
    for (i=0; i<Tnum; i++) {
        if ((BlockStart[i] <= code) && (BlockEnd[i] >= code)) {
            return n+(code-BlockStart[i]);
        }
        n += BlockEnd[i]-BlockStart[i]+1;
    }
    return (-1);
}

// ドットマトリックス用データへの変換
void DrawFont(int code) {
    int [] font = new int [8];
    int i, j;
    for (i=0; i<8; i++) {
        font[i] = FontData[18 + Tnum * 4 + code*8 + i]&0xff;
    }
    for (i=0; i<8; i++) {
        WordData[WordDataLen] = font[i];
        WordDataLen++;
    }
}

