import processing.serial.*;    // Arduinoと通信するためのライブラリを読み込む

Serial port;                   // シリアル通信を行うための変数の定義
float px, py;                  // ボールの位置
float vx, vy;                  // ボールの速度
float rx, rx1;                 // ラケットの現在位置と1ステップ前の位置

void setup()
{
    size(400, 400);                         // 400×400ドットの画面を作成
    port = new Serial(this, "COM3", 9600);  // 通信ポートと速度の設定
    frameRate(30);                          // 画面の更新頻度を30fps
    textSize(24);                           // 文字の大きさを24pt
    textAlign(CENTER);                      // 表示位置を中心
    px = 100;                               // ボール初期位置と初期速度
    py = 200;
    vx=2;
    vy=2;
    rx=width/2;                             // ラケットの初期位置
    rx1=rx;
    port.bufferUntil('\n');
    port.write('a');                        // 返信要求を送る
}

void draw()
{
    background(0);                      // 画面を黒で塗りつぶして更新
    noStroke();                         // 線なし
    fill(255);                          // 塗りつぶし色を白に
    rect(20, 20, width-40, height-20);  // ボールが動ける白い範囲を描く

    fill(0);                            // 塗りつぶし色を黒に
    rect(rx, height-20, 40, 20);        // ラケットの描画

    fill(127);                          // 塗りつぶし色を灰色に
    ellipse(px, py, 10, 10);            // ボールの描画

    if (px < 25)vx =- vx;               // 左の壁にぶつかったら横方向の速度を反転
    if (px > width-25)vx =- vx;         // 右の壁にぶつかったとき
    if (py < 25)vy =- vy;               // 上の壁にぶつかったら縦方向の速度を反転
    if (py > height-25) {               // ラケットとの衝突判定
        if (px > rx && px < rx+40)
            vy = -(abs(rx1-rx)*0.5+2);  // ラケットの移動速度によりボールの速さを変える
    }
    if (py > height) {                  // ラケットよりボールが下に行った場合
        text("Game Over", width/2, height/2);  // 文字の表示
        vy = 0.1;
    }
    if (py > height+9) {                // ボールがラケットの下に行ってから約3秒経過したか
        px = 100;                       // 初期値に戻す
        py = 200;
        vx = 2;
        vy = 2;
        rx = width/2;
        rx1 = rx;
    }
    px += vx;                           // ボールの位置の更新
    py += vy;
    rx1 = rx;                           // 1つ前のラケットの位置を保存
}

void serialEvent(Serial p) {
    String ms = port.readStringUntil('\n');  // 文字列の読み込み
    rx = int(trim(ms));                      // 改行コードの削除とint型への変換
    rx = map(rx, 0, 1023, 20, 340);          // ボリュームの角度によりラケットの位置を決める
    port.write('a');                         // 返信要求を送る
}

void mousePressed()
{
    port.write('a');                         // 返信要求を送る
}

