import processing.serial.*;         // Arduinoと通信するためのライブラリを読み込む
import kinect4WinSDK.Kinect;        // Kinectを使うためのライブラリを読み込む
import kinect4WinSDK.SkeletonData;  // Kinectで骨格抽出をするためのライブラリを読み込む

Kinect kinect;                      // Kinectを使うための変数
ArrayList <SkeletonData> bodies;    // 得られた体の関節位置を保存するための変数
Serial port;                        // シリアル通信を行うための変数

void setup()
{
    size(640, 480);                           // 640×480ドットの画面を作成
    port = new Serial(this, "COM3", 9600);    // 通信ポートと速度の設定
    kinect = new Kinect(this);                // Kinectを使うための設定
    bodies = new ArrayList<SkeletonData>();   // 骨格モデルのリストを設定
    port.write(60);                           // 初期角度を60度にする
    smooth();
}

void draw()
{
    background(0);
// image(kinect.GetImage(), 0, 0, 640, 480);  // カメラ画像の表示
// image(kinect.GetDepth(), 0, 0, 640, 480);  // 深度画像
// image(kinect.GetMask(), 0, 0, 640, 480);   // 人間の部分だけを表示
    for (int i=0; i<bodies.size (); i++)
    {
        drawSkeleton(bodies.get(i));          // 骨格を表示
        drawPosition(bodies.get(i));          // IDを表示
        SendAngle(bodies.get(i), Kinect.NUI_SKELETON_POSITION_HEAD, Kinect.NUI_SKELETON_POSITION_SHOULDER_CENTER);  // 頭と肩の中心の位置から頭の傾きを計算する。
    }
}

void SendAngle(SkeletonData _s, int _j1, int _j2)
{
    double sd;
    int angle;
    // 頭と肩の中心を結ぶ線の表示
    if (_s.skeletonPositionTrackingState[_j1] != Kinect.NUI_SKELETON_POSITION_NOT_TRACKED && _s.skeletonPositionTrackingState[_j2] != Kinect.NUI_SKELETON_POSITION_NOT_TRACKED) {
        sd = atan2(-(_s.skeletonPositions[_j1].y-_s.skeletonPositions[_j2].y)*height,(_s.skeletonPositions[_j1].x-_s.skeletonPositions[_j2].x)*width);  // 角度の計算
        angle = (int)(sd/PI*180.0);           // rad角からdeg角
        println("angle=", angle);             // コンソールに傾きを表示
        port.write(angle);                    // Arduinoに送信
    }
}
// 認識した人間に付くIDを画面上に表示（サンプル通り）
void drawPosition(SkeletonData _s)
{
    noStroke();
    fill(0, 100, 255);                        // 塗りつぶし色を青
    String s1 = str(_s.dwTrackingID);
    text(s1, _s.position.x*width/2, _s.position.y*height/2);
}
// 認識した人間の骨格モデルを描く（サンプル通り）
void drawSkeleton(SkeletonData _s)
{
    // Body
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_HEAD,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_CENTER);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_CENTER,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_LEFT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_CENTER,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_RIGHT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_CENTER,
    Kinect.NUI_SKELETON_POSITION_SPINE);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_LEFT,
    Kinect.NUI_SKELETON_POSITION_SPINE);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_RIGHT,
    Kinect.NUI_SKELETON_POSITION_SPINE);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SPINE,
    Kinect.NUI_SKELETON_POSITION_HIP_CENTER);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_HIP_CENTER,
    Kinect.NUI_SKELETON_POSITION_HIP_LEFT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_HIP_CENTER,
    Kinect.NUI_SKELETON_POSITION_HIP_RIGHT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_HIP_LEFT,
    Kinect.NUI_SKELETON_POSITION_HIP_RIGHT);

    // Left Arm
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_LEFT,
    Kinect.NUI_SKELETON_POSITION_ELBOW_LEFT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_ELBOW_LEFT,
    Kinect.NUI_SKELETON_POSITION_WRIST_LEFT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_WRIST_LEFT,
    Kinect.NUI_SKELETON_POSITION_HAND_LEFT);

    // Right Arm
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_SHOULDER_RIGHT,
    Kinect.NUI_SKELETON_POSITION_ELBOW_RIGHT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_ELBOW_RIGHT,
    Kinect.NUI_SKELETON_POSITION_WRIST_RIGHT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_WRIST_RIGHT,
    Kinect.NUI_SKELETON_POSITION_HAND_RIGHT);

    // Left Leg
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_HIP_LEFT,
    Kinect.NUI_SKELETON_POSITION_KNEE_LEFT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_KNEE_LEFT,
    Kinect.NUI_SKELETON_POSITION_ANKLE_LEFT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_ANKLE_LEFT,
    Kinect.NUI_SKELETON_POSITION_FOOT_LEFT);

    // Right Leg
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_HIP_RIGHT,
    Kinect.NUI_SKELETON_POSITION_KNEE_RIGHT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_KNEE_RIGHT,
    Kinect.NUI_SKELETON_POSITION_ANKLE_RIGHT);
    DrawBone(_s,
    Kinect.NUI_SKELETON_POSITION_ANKLE_RIGHT,
    Kinect.NUI_SKELETON_POSITION_FOOT_RIGHT);
}

// 骨格を表示（変更点あり）
void DrawBone(SkeletonData _s, int _j1, int _j2)
{
    noFill();
    stroke(255, 255, 0);
    if (_s.skeletonPositionTrackingState[_j1] != Kinect.NUI_SKELETON_POSITION_NOT_TRACKED &&
            _s.skeletonPositionTrackingState[_j2] != Kinect.NUI_SKELETON_POSITION_NOT_TRACKED) {
        line(_s.skeletonPositions[_j1].x*width,
            _s.skeletonPositions[_j1].y*height,
            _s.skeletonPositions[_j2].x*width,
            _s.skeletonPositions[_j2].y*height);
    }
}

// （サンプル通り）
void appearEvent(SkeletonData _s)
{
    if (_s.trackingState == Kinect.NUI_SKELETON_NOT_TRACKED)
    {
        return;
    }
    synchronized(bodies) {
        bodies.add(_s);
    }
}

// （サンプル通り）
void disappearEvent(SkeletonData _s)
{
    synchronized(bodies) {
        for (int i=bodies.size ()-1; i>=0; i--)
        {
            if (_s.dwTrackingID == bodies.get(i).dwTrackingID)
            {
                bodies.remove(i);
            }
        }
    }
}

// （サンプル通り）
void moveEvent(SkeletonData _b, SkeletonData _a)
{
    if (_a.trackingState == Kinect.NUI_SKELETON_NOT_TRACKED)
    {
        return;
    }
    synchronized(bodies) {
        for (int i=bodies.size ()-1; i>=0; i--)
        {
            if (_b.dwTrackingID == bodies.get(i).dwTrackingID)
            {
                bodies.get(i).copy(_a);
                break;
            }
        }
    }
}

// おまけ（sキーを押すと）座標データをdata.txtに保存
void keyPressed()
{
    if(key == 's') {                          // sキーが押されていれば
        PrintWriter writer;                   // ファイルを書き出すための変数を定義
        writer = createWriter("data.txt");    // 書き込みファイルの設定
        for(int i=0; i<bodies.size (); i++)   // 複数の人間が認識されていればその数だけ
        {
            SkeletonData _s = bodies.get(i);
            for(int j=0; j<kinect.NUI_SKELETON_POSITION_COUNT; j++) {  // すべての角度
                print(j + "\t" + _s.skeletonPositionTrackingState[j] + "\t" + _s.skeletonPositions[j].x + "\t" + _s.skeletonPositions[j].y + "\t" + _s.skeletonPositions[j].z + "\n");
                writer.print(j + "\t" + _s.skeletonPositionTrackingState[j] + "\t" + _s.skeletonPositions[j].x + "\t" + _s.skeletonPositions[j].y + "\t" + _s.skeletonPositions[j].z + "\n");
            }
        }
        writer.flush();                       // ファイル書き出し後処理
        writer.close();                       // ファイルを閉じる
    }
}

