import processing.serial.*;       // Arduinoと通信するためのライブラリを読み込む
import de.voidplus.leapmotion.*;  // Leap Motionを使うためのライブラリを読み込む

Serial port;                      // シリアル通信を行うための変数の定義
LeapMotion leap;                  // Leap Motionを使うための変数の定義
String gs = "Start";              // 認識した手の位置を表示するための変数を定義
String hp[] = {                   // 手の位置を表示するための4つの単語
    "None", "Left", "Center", "Right"
};
int hpn;                          // 手の位置を表す番号
int LeftLED, CenterLED, RightLED; // 左，中央，右のLEDの状態を保存しておく変数

void setup() {
    size(320, 240);                             // 320×240ドットの画面を作成
    port = new Serial(this, "COM3", 9600);      // 通信ポートと速度の設定
    textSize(24);                               // 文字のサイズを24pt
    leap = new LeapMotion(this).withGestures(); // Leap Motionをジェスチャー付きで使用
    LeftLED = CenterLED = RightLED = 0;         // 初期状態ではLEDが消灯
    port.write('a');                            // すべて消灯させる指令を送る
}

void draw() {
    background(255);                        // 画面を白で塗りつぶして更新
    hpn = 0;

    for (Hand hand : leap.getHands ()) {    // 認識した手の数だけ処理を行う
        fill(0);
        PVector hand_position = hand.getPosition();
        text("("+hand_position.x+")", 150, 50);
        fill(192);
        hand.draw();                        // 手のひらの位置を表示
        if(hand_position.x < 120) {         // 左の方だったら
            hpn = 1;
        } else if(hand_position.x < 180) {  // 中央付近だったら
            hpn = 2;
        } else {                            // 右の方だったら
            hpn = 3;
        }
        for (Finger finger : hand.getFingers ()) {
            finger.draw();                  // 指先の位置を表示
        }
    }
    stroke(0);
    fill(LeftLED);                          // 左のLEDの状態を表示
    ellipse(60, 200, 30, 30);
    fill(CenterLED);                        // 中央のLEDの状態を表示
    ellipse(160, 200, 30, 30);
    fill(RightLED);                         // 右のLEDの状態を表示
    ellipse(260, 200, 30, 30);

    fill(0);                                // 文字の色を黒
    text(gs, 50, 20);                       // ジェスチャーを文字で表示
    text(hp[hpn], 50, 50);                  // 手のひらの位置を文字で表示
}
// 手を払うようなジェスチャーをしたときに呼び出される関数
void leapOnSwipeGesture(SwipeGesture g, int state) {
    gs = "SwipeGesture";
    println(gs + ":" + hp[hpn]);
    LeftLED = 0;                            // LEDをすべて消す
    CenterLED = 0;
    RightLED = 0;
    port.write('a');
}
// 画面に触れるようなジェスチャーをしたときに呼び出される関数
void leapOnScreenTapGesture(ScreenTapGesture g) {
    gs = "ScreenTapGesture";
    println(gs + ":" + hp[hpn]);
    LeftLED = 255;                          // LEDをすべて光らせる
    CenterLED = 255;
    RightLED = 255;
    port.write('A');
}
// キーを押すようなジェスチャーをしたときに呼び出される関数
void leapOnKeyTapGesture(KeyTapGesture g) {
    gs = "KeyTapGesture";
    println(gs + ":" + hp[hpn]);
    if(hpn == 1) {                          // 手の位置によって1つだけLEDを消す
        LeftLED = 0;
        port.write('l');
    }else if (hpn == 2) {
        CenterLED = 0;
        port.write('c');
    }else if (hpn == 3) {
        RightLED = 0;
        port.write('r');
    }
}
// 手を回すようなジェスチャーをしたときに呼び出される関数
void leapOnCircleGesture(CircleGesture g, int state) {
    gs = "CircleGesture";
    println(gs + ":" + hp[hpn]);
    if(hpn == 1) {                          // 手の位置によって1つだけLEDを光らせる
          LeftLED = 255;
          port.write('L');
    }else if (hpn == 2) {
        CenterLED = 255;
        port.write('C');
    }else if (hpn == 3) {
        RightLED = 255;
        port.write('R');
    }
}

