import processing.serial.*;   // Arduinoと通信するためのライブラリを読み込む
import gab.opencv.*;          // OpenCVを使うためのライブラリを読み込む
import processing.video.*;    // カメラを使うためのライブラリを読み込む
import java.awt.*;

Serial port;                  // シリアル通信を行うための変数の定義
Capture video;                // カメラを使うための変数の定義
OpenCV opencv;                // OpenCVを使うための変数の定義
float angle;                  // サーボモーターの角度を保存する変数の定義

void setup() {
size(640, 480);                           // 640×480ドットの画面を作成
port = new Serial(this, "COM3", 9600);    // 通信ポートと速度の設定
video = new Capture(this, 640/2, 480/2);  // カメラの設定
opencv = new OpenCV(this, 640/2, 480/2);  // OpenCVの設定
opencv.loadCascade(OpenCV.CASCADE_FRONTALFACE);
video.start();                            // カメラからの情報取得をスタート
angle = 60;                               // 初期角度を60度
}

void draw() {
    scale(2);                             // カメラから得られた画像を2倍に
    image(video, 0, 0 );                  // カメラ画像を画面に表示
    opencv.loadImage(video);              // カメラ画像をOpenCVで使うための準備

    Rectangle[] faces = opencv.detect();  // 顔認識
    noFill();                             // 塗りつぶしなし
    stroke(0, 255, 0);                    // 線の色を緑
    strokeWeight(3);                      // 線の太さ3
    for(int i = 0; i<faces.length; i++) { // 認識した顔を四角で囲む
        rect(faces[i].x, faces[i].y, faces[i].width, faces[i].height);
    }
    if(faces.length == 1) {
        int fx = faces[0].x+faces[0].width/2;
        if(fx > width/4+5){               // 画面の中心より5ピクセル以上右にあれば
            angle -= 0.5;                 // サーボモーターの角度を小さくする
            if(angle < 0)angle = 0;
        }else if(fx < width/4-5){         // 画面の中心より5ピクセル以上左にあれば
            angle += 0.5;                 // サーボモーターの角度を大きくする
            if (angle>120)angle=120;
        }
        port.write(byte(angle));          // サーボモーターの角度を送る
    }
}

    void captureEvent(Capture c) {
    c.read();
}

void mousePressed()
{
    angle = 60;                    // 初期角度を60度
    port.write(angle);             // Arduinoにサーボモーターの角度を送る
}

